/***********************************************************************

	rtc.c


	real time clock modeule

***********************************************************************/

#include "GenericTypeDefs.h"
#include "HardwareProfile.h"
#include <stdio.h>
#include <stdlib.h>
//#define	DEBUG_MODE
#ifdef DEBUG_MODE
	#include "uart2.h"
#endif

static	volatile unsigned int timer_tick;	// internal variable
      	volatile unsigned int timer_val;	// global variable for Application Timer, 

static	volatile unsigned long capture_interval_time; // capture interval constants
static	volatile unsigned long interval_timer_val; // capture time flag
		volatile unsigned int  is_capture;

static	volatile unsigned int 	year;
static	volatile unsigned char	leap, month, day, hour, minute, second;
static	const unsigned char	mdays[] = { 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31,}; 

int SetClockVars (unsigned int year, unsigned char month, unsigned char day,
				unsigned char hour, unsigned char minute, unsigned char second);

void	rtc_set(char *date_time);

/*----------------------------------------------------------------------
	initialize timer modeule
----------------------------------------------------------------------*/

void rtc_init(void)
{
#ifdef DEBUG_MODE
	UART2PrintString("rtc_init()\r\n");
#endif

	rtc_set("2012/07/01 09:00:00");
	
	IEC0bits.T1IE = 1;	// enable T1 interrupt
	IFS0bits.T1IF = 0;	// clear interrupt flag
	IPC0bits.T1IP = 4;	// priority is level 4.

	//--- set interval@10ms timer
	T1CON = 0x0020;	// use timer(T1), set prescale 1:64
	TMR1 = 0;
	PR1 = 2499;		// 10msec (2500 * 64 = 160000 = 160KHz, 16M/0.16M=100Hz,10msec)
	T1CONbits.TON = 1;	// start T2
}


/*----------------------------------------------------------------------
	set file attribute time
----------------------------------------------------------------------*/

void	set_fattime(void)
{
#ifdef DEBUG_MODE
	UART2PrintString("set_fattime()\r\n");
#endif
	asm("DISI #200");
	SetClockVars(year, month, day, hour, minute, second);
	asm("DISI #0");
}

/*----------------------------------------------------------------------
	set capture time
----------------------------------------------------------------------*/

void	set_capture_time(unsigned int interval_time)
{
	asm("DISI #200");
	capture_interval_time = interval_time * 100UL;
	interval_timer_val = capture_interval_time;
	is_capture = 0;
	asm("DISI #0");
}


/*----------------------------------------------------------------------
	timer set
	rtc_set("2012/09/01 07:52:05");
----------------------------------------------------------------------*/

void	rtc_set(char *date_time)
{
	int	y, m,d, hh,mm,ss;
#ifdef DEBUG_MODE
	UART2PrintString("rtc_set()\r\n");
#endif

	y = atoi(date_time);
	m = atoi(date_time+5);
	d = atoi(date_time+8);
	hh = atoi(date_time+11);
	mm = atoi(date_time+14); 
	ss = atoi(date_time+17);

	leap = (y % 4 == 0) && (y % 100 != 0 || y % 400 == 0);

	asm("DISI #200");
	year = y; month = m; day = d;
	hour = hh; minute = mm; second = ss;
	asm("DISI #0");

}

/*----------------------------------------------------------------------
	get_rtcstr(buf)
	 write to buf, "2012/09/01 07:52:05"
----------------------------------------------------------------------*/

void	get_rtcstr(char *buf)
{
	asm("DISI #2000");
	sprintf(buf, "%4d/%02d/%02d %02d:%02d:%02d",
		year, month, day, hour, minute, second);
	asm("DISI #0");
}


/*----------------------------------------------------------------------
	Timer T1 interrupt
	Timer T1 10msec interval timer.
----------------------------------------------------------------------*/

void __attribute__ ((__interrupt__, no_auto_psv)) _T1Interrupt(void)
{
	IFS0bits.T1IF = 0;		// clear interrupt flag

	// count down application timer value
	if(timer_val) --timer_val;

	if(interval_timer_val){
		--interval_timer_val;
	}else{
		if(hour >= 4 && hour <= 20){
			is_capture = 1;
		}else{
			is_capture = 0;
		}
		interval_timer_val = capture_interval_time;
	}


	// real timer clock count up
	if(++timer_tick >= 100){
	  timer_tick = 0;
	  if(++second >= 60){
	    second = 0;
	    if(++minute >= 60){
	      minute = 0;
	      if(++hour >= 24){
	        hour = 0;
	        if(++day > (month == 2 && leap ? 29: mdays[month-1]) ){         
	          day = 1;
	          if(++month > 12){
	            month = 1;
	            year++;
				leap = (year % 4 == 0) && (year % 100 != 0 || year % 400 == 0);
	          } // month
	        } // day
	      } // hour
	    } // minute
	  } // second
	} // tick  
}


/*** end of rtc.c ****************************************************/
